function info = MOtestSoftICA()
    ns = [4, 16, 64, 256, 1024, 2048];
    ps = [2, 8,  32, 128, 512, 1024];
    nums = [128, 128, 32, 8, 2, 2];
    numruns = 50;
    tabtime = zeros(numruns, length(ns));
    tabiter = zeros(numruns, length(ns));
    tabnumf = zeros(numruns, length(ns));
    
    for k = 1 : length(ns)
        n = ns(k);
        p = ps(k);
        num = nums(k);
        for h = 1 : numruns
            rand('twister', h - 1);
            C = rand(n, n, num);
            for i = 1 : num
                C(:, :, i) = C(:, :, i) + C(:, :, i)';
            end
            tmpx0 = rand(n, p);
            [x0, ~] = qr(tmpx0, 0);


            manifold = stiefelfactory(n, p, 1);
            problem.M = manifold;

            problem.cost = @(x, store)f(x, store, C);
            problem.egrad = @(x, store)gf(x, store, C);

            % checkgradient(problem);
        %     options.tolgradnorm = 1e-1;
            options.maxiter = 30;
            options.verbosity = 1;
            options.ls_optimism = 1;
            [x, xcost, info, options] = steepestdescent(problem, x0, options);
            tabtime(h, k) = info(end).time;
            tabiter(h, k) = length(info);
            cn = 1;
            for i = 2 : options.maxiter + 1; 
                cn = cn + info(i).linesearch.costevals;
            end
            tabnumf(h, k) = cn;
        end
    end
    dlmwrite('Manopt_time', tabtime);
    dlmwrite('Manopt_iter', tabiter);
    dlmwrite('Manopt_numf', tabnumf);
end

function [output, store] = f(x, store, C)
    [n, p] = size(x);
    num = size(C, 3);
    [m,n,r] = size(C);
    store.CY = permute(reshape(reshape(permute(C,[1 3 2]),[],n)*x,m,r,[]),[1 3 2]);
    store.D = sum(repmat(x, 1, num) .* reshape(store.CY, [], p * num));
    output = - norm(store.D)^2;
end
    
function [output, store] = gf(x, store, C)
    if(~isfield(store, 'D'))
        [~, store] = f(x, store, C);
    end
    [n, p] = size(x);
    num = size(C, 3);
    output = -4 * sum(reshape(repmat(store.D, 1, n), p, num, n) .* permute(store.CY, [2, 3, 1]), 2);
    output = reshape(output, p, n)';
end
