import autograd.numpy as np

from pymanopt.manifolds import Stiefel
from pymanopt import Problem
from pymanopt.solvers import SteepestDescent
from pymanopt.solvers.linesearch import LineSearchBackTracking

ns = [4, 16, 64, 256, 1024, 2048]
ps = [2,  8, 32, 128, 512,  1024]
nums = [128, 128, 32, 8, 2, 2];
numruns = 50
tabtime = np.zeros((numruns, len(ns)))
tabiter = np.zeros((numruns, len(ns)))

for k in range(0, len(ns)):
    # (1) Instantiate a manifold
    n = ns[k]
    p = ps[k]
    manifold = Stiefel(n, p)
    num = nums[k]
    for h in range(0, numruns):
        print(n, p, num)
        np.random.seed(h) # the same as matlab: rand('twister', 1);
        C = np.random.rand(num, n, n)
        for i in range(0, num):
            C[i, :, :] = C[i, :, :].T + C[i, :, :]

        tmpx0 = np.random.rand(p, n).T

        x0, r = np.linalg.qr(tmpx0)

        CY = np.zeros((num, n, p))
        DD = np.zeros(p * num)
        # (2) Define the cost function (here using autograd.numpy)
        def cost(Y):
            global CY, DD
            CY = np.tensordot(C, Y, 1)
            DD = np.sum(Y * CY, axis=1)
            DD = DD.reshape(p * num, 1)
            result = np.sum(DD**2)
            return -result
        def egrad(Y):
            global CY, DD
            egf = -4.0 * np.sum(np.multiply(np.tile(DD, (n, 1)).reshape(n, num, p), np.transpose(CY, (1, 0, 2))), axis = 1)
            return egf
        problem = Problem(manifold=manifold, cost=cost, egrad=egrad, verbosity = 1)

        # (3) Instantiate a Pymanopt solver
        solver = SteepestDescent(maxiter=31,logverbosity = 1,linesearch=LineSearchBackTracking(optimism=1))

        # let Pymanopt do the rest
        Xopt, optlog = solver.solve(problem, x=x0)
        tabtime[h, k] = optlog['final_values']['time']
        tabiter[h, k] = optlog['final_values']['iterations']

np.savetxt('Pymanopt_time', tabtime)
np.savetxt('Pymanopt_iter', tabiter)

#print(optlog)
print(tabtime)
print(tabiter)